@@ -114,7 +114,7 @@ for ext, warn in skippable_extensions:
 templates_path = ['_templates']
 
 # The suffix of source filenames.
-source_suffix = '.rst'
+source_suffix = {'.rst': 'restructuredtext'}
 
 # General substitutions.
 project = 'NumPy'
@@ -150,8 +150,26 @@ default_role = "autolink"
 exclude_dirs = []
 
 exclude_patterns = []
+suppress_warnings = []
+nitpick_ignore = []
+
 if sys.version_info[:2] >= (3, 12):
-    exclude_patterns += ["reference/distutils.rst"]
+    exclude_patterns += [
+        "reference/distutils.rst",
+        "reference/distutils/misc_util.rst",
+    ]
+    suppress_warnings += [
+        'toc.excluded',  # Suppress warnings about excluded toctree entries
+    ]
+    nitpicky = True
+    nitpick_ignore += [
+        ('ref', 'numpy-distutils-refguide'),
+        # The first ignore is not catpured without nitpicky = True.
+        # These three ignores are required once nitpicky = True is set.
+        ('py:mod', 'numpy.distutils'),
+        ('py:class', 'Extension'),
+        ('py:class', 'numpy.distutils.misc_util.Configuration'),
+    ]
 
 # If true, '()' will be appended to :func: etc. cross-reference text.
 add_function_parentheses = False
@@ -272,6 +290,10 @@ html_theme_options = {
         "json_url": "https://numpy.org/doc/_static/versions.json",
     },
     "show_version_warning_banner": True,
+    "analytics": {
+        "plausible_analytics_domain": "numpy.org/doc/stable/",
+        "plausible_analytics_url": ("https://views.scientific-python.org/js/script.js"),
+    },
 }
 
 html_title = f"{project} v{version} Manual"
@@ -612,7 +634,7 @@ breathe_default_project = "numpy"
 breathe_default_members = ("members", "undoc-members", "protected-members")
 
 # See https://github.com/breathe-doc/breathe/issues/696
-nitpick_ignore = [
+nitpick_ignore += [
     ('c:identifier', 'FILE'),
     ('c:identifier', 'size_t'),
     ('c:identifier', 'PyHeapTypeObject'),
