@@ -69,5 +69,10 @@ void VQSelect(uint16_t* HWY_RESTRICT keys, const size_t n, const size_t k,
   HWY_DYNAMIC_DISPATCH(SelectU16Asc)(keys, n, k);
 }
 
+void Sorter::operator()(uint16_t* HWY_RESTRICT keys, size_t n,
+                        SortAscending tag) const {
+  VQSort(keys, n, tag);
+}
+
 }  // namespace hwy
 #endif  // HWY_ONCE
